@@ -159,6 +159,7 @@ typedef struct {
 	POOL_TEMP_TABLE_STATE	state;	/* see above */
 }			POOL_TEMP_TABLE;
 
+
 /*
  * Per session context:
  */
@@ -290,6 +291,16 @@ typedef struct
 																* transaction has been
 																* started by a
 																* multi-statement-query */
+	/*
+	 * True if query cache feature disabled until session ends.
+	 * This is set when SET ROLE/SET SESSION AUTHORIZATION executed.
+	 */
+	bool	query_cache_disabled;
+	/*
+	 * True if query cache feature disabled until current transaction ends.
+	 * This is set when REVOKE executed in a transaction.
+	 */
+	bool	query_cache_disabled_tx;
 }			POOL_SESSION_CONTEXT;
 
 extern void pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend);
@@ -377,4 +388,11 @@ extern bool is_tx_started_by_multi_statement_query(void);
 extern void set_tx_started_by_multi_statement_query(void);
 extern void unset_tx_started_by_multi_statement_query(void);
 
+extern void set_query_cache_disabled(void);
+extern void unset_query_cache_disabled(void);
+extern bool query_cache_disabled(void);
+extern void set_query_cache_disabled_tx(void);
+extern void unset_query_cache_disabled_tx(void);
+extern bool query_cache_disabled_tx(void);
+
 #endif							/* POOL_SESSION_CONTEXT_H */
